# -*- coding: utf-8 -*-
"""These test the utils.py functions."""

from typing import Optional, Tuple, Union

import pytest
from hypothesis import given
from hypothesis.strategies import floats, integers
from natsort.ns_enum import NSType, ns
from natsort.utils import NumTransformer, parse_number_or_none_factory


@pytest.mark.usefixtures("with_locale_en_us")
@pytest.mark.parametrize(
    "alg, example_func",
    [
        (ns.DEFAULT, lambda x: ("", x)),
        (ns.PATH, lambda x: (("", x),)),
        (ns.UNGROUPLETTERS | ns.LOCALE, lambda x: (("xx",), ("", x))),
        (ns.PATH | ns.UNGROUPLETTERS | ns.LOCALE, lambda x: ((("xx",), ("", x)),)),
    ],
)
@given(x=floats(allow_nan=False, allow_infinity=False) | integers())
def test_parse_number_factory_makes_function_that_returns_tuple(
    x: Union[float, int], alg: NSType, example_func: NumTransformer
) -> None:
    parse_number_func = parse_number_or_none_factory(alg, "", "xx")
    assert parse_number_func(x) == example_func(x)


@pytest.mark.parametrize(
    "alg, x, result",
    [
        (ns.DEFAULT, 57, ("", 57)),
        (
            ns.DEFAULT,
            float("nan"),
            ("", float("-inf"), "1"),
        ),  # NaN transformed to -infinity
        (
            ns.NANLAST,
            float("nan"),
            ("", float("+inf"), "3"),
        ),  # NANLAST makes it +infinity
        (ns.DEFAULT, None, ("", float("-inf"), "2")),  # None transformed to -infinity
        (ns.NANLAST, None, ("", float("+inf"), "2")),  # NANLAST makes it +infinity
        (ns.DEFAULT, float("-inf"), ("", float("-inf"), "3")),
        (ns.NANLAST, float("+inf"), ("", float("+inf"), "1")),
    ],
)
def test_parse_number_factory_treats_nan_and_none_special(
    alg: NSType, x: Optional[Union[float, int]], result: Tuple[str, Union[float, int]]
) -> None:
    parse_number_func = parse_number_or_none_factory(alg, "", "xx")
    assert parse_number_func(x) == result



"""
Fixtures for pytest.
"""

import locale
from typing import Iterator

import hypothesis
import pytest
from natsort.compat.locale import dumb_sort


# This disables the "too slow" hypothesis heath check globally.
# For some reason it thinks that the text/binary generation is too
# slow then causes the tests to fail.
hypothesis.settings.register_profile(
    "slow-tests", suppress_health_check=[hypothesis.HealthCheck.too_slow]
)


def load_locale(x: str) -> None:
    """Convenience to load a locale."""
    locale.setlocale(locale.LC_ALL, str("{}.UTF-8".format(x)))


@pytest.fixture()
def with_locale_en_us() -> Iterator[None]:
    """Convenience to load the en_US locale - reset when complete."""
    orig = locale.getlocale()
    load_locale("en_US")
    yield
    locale.setlocale(locale.LC_ALL, orig)


@pytest.fixture()
def with_locale_de_de() -> Iterator[None]:
    """
    Convenience to load the de_DE locale - reset when complete - skip if missing.
    """
    orig = locale.getlocale()
    try:
        load_locale("de_DE")
    except locale.Error:
        pytest.skip("requires de_DE locale to be installed")
    else:
        yield
    finally:
        locale.setlocale(locale.LC_ALL, orig)


@pytest.fixture()
def with_locale_cs_cz() -> Iterator[None]:
    """
    Convenience to load the cs_CZ locale - reset when complete - skip if missing.
    """
    orig = locale.getlocale()
    try:
        load_locale("cs_CZ")
        if dumb_sort():
            pytest.skip("requires a functioning locale library to run")
    except locale.Error:
        pytest.skip("requires cs_CZ locale to be installed")
    else:
        yield
    finally:
        locale.setlocale(locale.LC_ALL, orig)



from typing import overload

@overload
def Locale() -> str: ...
@overload
def Locale(x: str) -> str: ...

class UCollAttribute:
    NUMERIC_COLLATION: int

class UCollAttributeValue:
    ON: int

class DecimalFormatSymbols:
    kGroupingSeparatorSymbol: int
    kDecimalSeparatorSymbol: int
    def __init__(self, locale: str) -> None: ...
    def getSymbol(self, symbol: int) -> str: ...

class Collator:
    @classmethod
    def createInstance(cls, locale: str) -> Collator: ...
    def getSortKey(self, source: str) -> bytes: ...
    def setAttribute(self, attr: int, value: int) -> None: ...



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""

from functools import partial
from typing import Any, Callable, FrozenSet, Union

import pytest
from hypothesis import assume, example, given
from hypothesis.strategies import floats, integers, text
from natsort.compat.fastnumbers import try_float, try_int
from natsort.compat.locale import get_strxfrm
from natsort.ns_enum import NSType, NS_DUMB, ns
from natsort.utils import groupletters, string_component_transform_factory

# There are some unicode values that are known failures with the builtin locale
# library on OSX and some other BSD-based systems that has nothing to do with
# natsort (a ValueError is raised by strxfrm). Let's filter them out.
try:
    bad_uni_chars = frozenset(chr(x) for x in range(0x10FEFD, 0x10FFFF + 1))
except ValueError:
    # Narrow unicode build... no worries.
    bad_uni_chars = frozenset()


def no_bad_uni_chars(x: str, _bad_chars: FrozenSet[str] = bad_uni_chars) -> bool:
    """Ensure text does not contain bad unicode characters"""
    return not any(y in _bad_chars for y in x)


def no_null(x: str) -> bool:
    """Ensure text does not contain a null character."""
    return "\0" not in x


def input_is_ok_with_locale(x: str) -> bool:
    """Ensure this input won't cause locale.strxfrm to barf"""
    # Bad input can cause an OSError if the OS doesn't support the value
    try:
        get_strxfrm()(x)
    except OSError:
        return False
    else:
        return True


@pytest.mark.parametrize(
    "alg, example_func",
    [
        (ns.INT, partial(try_int, map=True)),
        (ns.DEFAULT, partial(try_int, map=True)),
        (ns.FLOAT, partial(try_float, map=True, nan=float("-inf"))),
        (ns.FLOAT | ns.NANLAST, partial(try_float, map=True, nan=float("+inf"))),
        (ns.GROUPLETTERS, partial(try_int, map=True, on_fail=groupletters)),
        (ns.LOCALE, partial(try_int, map=True, on_fail=lambda x: get_strxfrm()(x))),
        (
            ns.GROUPLETTERS | ns.LOCALE,
            partial(
                try_int, map=True, on_fail=lambda x: get_strxfrm()(groupletters(x))
            ),
        ),
        (
            NS_DUMB | ns.LOCALE,
            partial(
                try_int, map=True, on_fail=lambda x: get_strxfrm()(groupletters(x))
            ),
        ),
        (
            ns.GROUPLETTERS | ns.LOCALE | ns.FLOAT | ns.NANLAST,
            partial(
                try_float,
                map=True,
                on_fail=lambda x: get_strxfrm()(groupletters(x)),
                nan=float("+inf"),
            ),
        ),
    ],
)
@example(x=float("nan"))
@example(x="Å")
@given(
    x=integers()
    | floats()
    | text().filter(bool).filter(no_bad_uni_chars).filter(no_null)
)
@pytest.mark.usefixtures("with_locale_en_us")
def test_string_component_transform_factory(
    x: Union[str, float, int], alg: NSType, example_func: Callable[[str], Any]
) -> None:
    string_component_transform_func = string_component_transform_factory(alg)
    x = str(x)
    assume(input_is_ok_with_locale(x))
    try:
        assert list(string_component_transform_func(x)) == list(example_func(x))
    except ValueError as e:  # handle broken locale lib on OSX.
        if "is not in range" not in str(e):
            raise



# -*- coding: utf-8 -*-
"""These test the utils.py functions."""

import unicodedata
from typing import Any, Callable, Iterable, List, Tuple, Union

import pytest
from hypothesis import given
from hypothesis.strategies import floats, integers, lists, text
from natsort.compat.fastnumbers import try_float
from natsort.ns_enum import NSType, NS_DUMB, ns
from natsort.utils import (
    FinalTransform,
    NumericalRegularExpressions as NumRegex,
    StrParser,
)
from natsort.utils import parse_string_factory


class CustomTuple(Tuple[Any, ...]):
    """Used to ensure what is given during testing is what is returned."""

    original: Any = None


def input_transform(x: Any) -> Any:
    """Make uppercase."""
    try:
        return x.upper()
    except AttributeError:
        return x


def final_transform(x: Iterable[Any], original: str) -> FinalTransform:
    """Make the input a CustomTuple."""
    t = CustomTuple(x)
    t.original = original
    return t


def parse_string_func_factory(alg: NSType) -> StrParser:
    """A parse_string_factory result with sample arguments."""
    sep = ""
    return parse_string_factory(
        alg,
        sep,
        NumRegex.int_nosign().split,
        input_transform,
        lambda x: try_float(x, map=True),
        final_transform,
    )


@given(x=floats() | integers())
def test_parse_string_factory_raises_type_error_if_given_number(
    x: Union[int, float]
) -> None:
    parse_string_func = parse_string_func_factory(ns.DEFAULT)
    with pytest.raises(TypeError):
        assert parse_string_func(x)  # type: ignore


# noinspection PyCallingNonCallable
@pytest.mark.parametrize(
    "alg, orig_func",
    [
        (ns.DEFAULT, lambda x: x.upper()),
        (ns.LOCALE, lambda x: x.upper()),
        (ns.LOCALE | NS_DUMB, lambda x: x),  # This changes the "original" handling.
    ],
)
@given(
    x=lists(
        elements=floats(allow_nan=False) | text() | integers(), min_size=1, max_size=10
    )
)
@pytest.mark.usefixtures("with_locale_en_us")
def test_parse_string_factory_invariance(
    x: List[Union[float, str, int]], alg: NSType, orig_func: Callable[[str], str]
) -> None:
    parse_string_func = parse_string_func_factory(alg)
    # parse_string_factory is the high-level combination of several dedicated
    # functions involved in splitting and manipulating a string. The details of
    # what those functions do is not relevant to testing parse_string_factory.
    # What is relevant is that the form of the output matches the invariant
    # that even elements are string and odd are numerical. That each component
    # function is doing what it should is tested elsewhere.
    value = "".join(map(str, x))  # Convert the input to a single string.
    result = parse_string_func(value)
    result_types = list(map(type, result))
    expected_types = [str if i % 2 == 0 else float for i in range(len(result))]
    assert result_types == expected_types

    # The result is in our CustomTuple.
    assert isinstance(result, CustomTuple)

    # Original should have gone through the "input_transform"
    # which is uppercase in these tests.
    assert result.original == orig_func(unicodedata.normalize("NFD", value))



#! /usr/bin/env python

from setuptools import find_packages, setup

setup(
    name="natsort",
    version="8.4.0",
    packages=find_packages(),
    entry_points={"console_scripts": ["natsort = natsort.__main__:main"]},
    python_requires=">=3.7",
    extras_require={"fast": ["fastnumbers >= 2.0.0"], "icu": ["PyICU >= 1.0.0"]},
    package_data={"": ["py.typed"]},
    zip_safe=False,
)
